var allCalls = []; // All calls in the system.
var moreCalls = true; // Used to get out of the promise.
var vector = 0; // The call vector for the current user.

/**
 * Load all calls.
 * 
 * @param Object data
 *   Entity information.
 * @return []
 */
function getCalls(data)
{
	ZOHO.CRM.API.getRelatedRecords({Entity: data.Entity, RecordID: data.EntityId, RelatedList: 'Calls'}).then(function(calls){
		if ( calls.data !== undefined )
		{
			displayCalls(calls);
			//if ( data.EntityId == "294376000006875312")
			//{
				getCallRecords(1, data, calls);
			//}
		}
	});
}

/**
 * Display the calls on the graph.
 * 
 * @param Object[] calls
 *   Call objects.
 * @return void
 */
function displayCalls(calls)
{
	for ( var i = 0; i < calls.data.length; i++ )
	{
		var call = calls.data[i];

		// Only pass the calls that are outbound.
		if ( call.Call_Type == 'Outbound' )
		{
			var timeString = call.Call_Start_Time.substr(11,5);
			var hour = parseInt(timeString.substr(0,2),10);
			var min = parseInt(timeString.substr(3,2),10);
			var position = (hour-8 + min/60) / 13 * 100 + 7.69/2;

			var style ='color:green;z-index:2;margin-top:-27px;';
			var title = 'Gesproken';
			var content = '<span style="font-size:64px;">&bull;</span>';
			switch ( call.Call_Result )
			{
			case 'No response/Busy':
				style = 'top:9px;color:#661900;z-index:1;margin-top:auto;';
				title ='Bezet/niet bereikt';
				content = 'x';
				break;
			case 'Geen gehoor':
				style = 'top:9px;color:#e35d62;z-index:1;margin-top:auto;';
				title = 'Geen gehoor';
				content = 'x';
				break;
			case 'SMS gestuurd':
				style = 'top:18px;color:#6BE080;z-index:1;margin-top:auto;';
				title = 'SMS gestuurd';
				content = 's';
				break;
			case 'Voicemail':
				style = 'top:9px;color:#e35ddf;z-index:1;margin-top:auto;';
				title = 'Voicemail';
				content = 'v';
				break;
			case null:
				style = 'top:18px;color:#000;z-index:1;margin-top:auto;';
				title = 'Toekomstig (' + call.Call_Start_Time.substr(8,2) + call.Call_Start_Time.substr(4,4) + call.Call_Start_Time.substr(0,4) + ')';
				content = '&gt;';
				break;
			}
			$('#callhistory').append('<div class="call" style="left:' + position + '%;' + style + '" title="' + title + '">' + content + '</div>');
		}
		
	}
}

function getCallRecords(page, data, calls)
{
	ZOHO.CRM.API.getAllRecords({Entity:"Calls",sort_order:"asc",per_page:200,page:page})
	.then(function(data2){
		if ( data2.info != undefined )
		{
			var done = data2.info.more_record;
			allCalls = allCalls.concat(data2.data);
		    if ( !done )
		    {
		    	getCallRecords(page + 1, data, calls);
		    }
		}
	    else
	    {
	    	// Propose a timeslot.
	    	proposeCall(data,calls);
	    }
	});
}

/**
 * Propose a new call time and show it on the graph.
 * 
 * @param Object data
 * 	 Entity information.
 * @param Object[] calls
 *   Call objects.
 * @return void
 */
function proposeCall(data, calls)
{
	// Create a vector for the call of this entity.
	vector = [0,0,0,0,0,0,0,0,0,0,0,0,0];
	calls.data.forEach(function(call)
	{
		
		var timeString = call.Call_Start_Time.substr(11,5);
		var hour = parseInt(timeString.substr(0,2),10) - 8;
		if ( hour >= 0 && hour < 13 )
		{
			if ( call.Call_Result == 'No response/Busy' || call.Call_Result == 'Voicemail' || call.Call_Result == 'Geen gehoor' )
			{
				vector[hour] = 1;
			}
		}
	});

	// Get all calls from each entity in the module, except for the current entity.
	var vectors = [];
	allCalls.forEach(function(call)
	{
		if ( call.Who_Id != undefined && call.Who_Id.id != data.Entity_Id )
		{
			var id = call.Who_Id.id;
			if ( vectors[id] == undefined )
			{
				vectors[id] = {v: [0,0,0,0,0,0,0,0,0,0,0,0,0], age: 0, hit: 0};
			}
			var timeString = call.Call_Start_Time.substr(11,5);
			var hour = parseInt(timeString.substr(0,2),10) - 8;
			if ( hour >= 0 && hour < 13 )
			{
				if ( call.Call_Result == 'No response/Busy' || call.Call_Result == 'Voicemail' || call.Call_Result == 'Geen gehoor' )
				{
					vectors[id].v[hour] = 1;
				}
				else {
					vectors[id].hit = hour + 8;
				}
			}
		}
	});
	
	// Now that we have all vectors of all calls, find and count the matching ones.
	var hits = [];
	Object.keys(vectors).forEach(function(n)
	{
		v = vectors[n];
		var d = distance(v.v, vector);
		// Only keep and count the ones with a hit hour and a distance of 0 or 1.
		if ( v.hit >= 8 && d <= 2 )
		{
			if ( hits[v.hit] == undefined ) hits[v.hit] = 0;
			hits[v.hit]++;
		}
	});

	if ( hits.length > 0 )
	{
		// Find the item with the most hist.
		var start = 0;
		var minHit = 0;
		hits.forEach(function(hit, hour) {
			if ( hit > minHit )
			{
				start = hour;
				minHit = hit;
			}
		});
		var end = start + 1;
		start = ('0' + start).substr(-2);
		end = ('0' + end).substr(-2);
		$('#callproposal').html('Probeer eens tussen ' + start + '.00 en ' + end + '.00 uur te bellen.');
		var record = ZOHO.CRM.API.updateRecord({
			Entity: data.Entity,
			APIData: {
				"id": data.EntityId,
				"CallWizard_beste_beltijd": start + '.00 - ' + end + '.00'
			},
			Trigger: ["workflow"]
		});
	}
	else
	{
		$('#callproposal').html('Geen suggestie gevonden.');
	}
}
function distance(v1, v2)
{
	d = 0;
	v1.forEach(function(e, n)
	{
		d += Math.abs(e - v2[n]);
	});
	return d;
}

/**
 * Initialize the widget. Called once.
 * 
 * This function implements the PageLoad event handler,
 * initialized the widget and
 * registers the button click event handler. 
 * 
 * @returns void
 */
function initializeWidget()
{
	/*
	 * Subscribe to the EmbeddedApp onPageLoad event before initializing the widget 
	 */
	ZOHO.embeddedApp.on("PageLoad",function(data)
	{
		ZOHO.CRM.UI.Resize({height:"120"});
		
	 	// Verify if EntityInformation is passed.
		if(data && data.Entity)
		{
			// Get all calls and process them.
			getCalls(data);
		}

	});
	
	/*
	 * initialize the widget.
	 */
	ZOHO.embeddedApp.init();
	
}
